(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: PersonalFirewallDatatype.thy 8881 2010-04-02 09:37:05Z brucker $ *)

theory 
  PersonalFirewallDatatype
imports 
  FWTesting
begin

profiling_on


text{* The most basic firewall scenario; there is a personal PC on one
side and the Internet on the other.  There are two policies: the first
one allows all traffic from the PC to the Internet and denies all
coming into the PC. The second policy only allows specific ports from
the PC. This scenario comes in three variants: the first one specifies
the allowed protocols directly, the second together with their
respective port numbers, the third one only with the port numbers.  *}

datatype Adr = pc | internet

types DatatypeTwoNets = "Adr \<times> int"

instance Adr::adr ..

defs (overloaded)
 src_port_def: "src_port (x::(DatatypeTwoNets,'b) packet) \<equiv> snd (src x)"
 dest_port_def: "dest_port (x::(DatatypeTwoNets,'b) packet) \<equiv> snd (dest x)"

constdefs
 PC :: "DatatypeTwoNets net"
"PC \<equiv> {{(a,b). a = pc}}"

 Internet :: "DatatypeTwoNets net"
"Internet \<equiv> {{(a,b). a = internet}}"


text{*
Definition of the testing constraint
*}

constdefs
 not_in_same_net :: "(DatatypeTwoNets,DummyContent) packet \<Rightarrow> bool"
"not_in_same_net x \<equiv> (src x \<sqsubset> PC \<longrightarrow> dest x \<sqsubset> Internet) \<and> (src x \<sqsubset> Internet \<longrightarrow> dest x \<sqsubset> PC)" 

text {*
Definitions of the policies 

In fact, the short definitions wouldn't have to be written down - they
are the automatically simplified versions of their big counterparts.

*}

constdefs
 strictPolicy :: "(DatatypeTwoNets,DummyContent) Policy"
"strictPolicy \<equiv> deny_all ++ allow_all_from_to PC Internet"

 PortPolicy :: "(DatatypeTwoNets,'b) Policy"
"PortPolicy \<equiv> deny_all ++ allow_from_ports_to {80::port,24,21} PC Internet"


 PortPolicyBig :: "(DatatypeTwoNets,'b) Policy"
"PortPolicyBig \<equiv> deny_all ++ allow_from_port_to (80::port) PC Internet++ allow_from_port_to (24::port) PC Internet++ allow_from_port_to (21::port) PC Internet"

lemmas policyLemmas = strictPolicy_def PortPolicy_def PC_def Internet_def PortPolicyBig_def src_def dest_def src_port_def dest_port_def  
                      PolicyCombinators PortCombinators in_subnet_def

test_spec "not_in_same_net x \<longrightarrow> FUT x = strictPolicy x"
apply  (prepare_fw_spec)
apply (simp add: policyLemmas)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas)
store_test_thm "strict"

gen_test_data "strict"
thm strict.test_data

test_spec "not_in_same_net x \<longrightarrow> FUT x = PortPolicy x"
apply (prepare_fw_spec)
apply (simp add: policyLemmas)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas)
store_test_thm "port"

gen_test_data "port"
thm port.test_data

test_spec "not_in_same_net x \<longrightarrow> FUT x = PortPolicyBig x"
apply (prepare_fw_spec)
apply (simp add: policyLemmas)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas)
store_test_thm "portBig"

gen_test_data "portBig"
thm portBig.test_data


write_clocks "time_stats_PersonalFirewallDatatype.tex"

profiling_off

end
