(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *
 * FWTesting.thy ---
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany
 *               2009-2010 Univerisite Paris-Sud, France
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************.***************************************************************)
(* $Id: FTP_Test.thy 8881 2010-04-02 09:37:05Z brucker $ *)

header{*Test Data for FTP*}
theory FTP_Test
imports 
  FWTesting
begin


text{* In this theory we generate the test data for correct runs of
the FTP protocol. As usual, we start with definining the networks and
the policy. We use a rather simple policy which allows only FTP
connections starting from the Intranet and going to the Internet, and
deny everything else. *}

constdefs 
  intranet :: "IntegerPort net"
  "intranet \<equiv> {{(a,e) . a = 3}}"

  internet :: "IntegerPort net"
  "internet \<equiv> {{(a,c). a > 3}}"

constdefs 
  ftp_policy :: "(IntegerPort,ftp_msg) Policy"
  "ftp_policy \<equiv> deny_all ++ allow_from_to_port (21::port) intranet  internet"


text{* The next two constants check if an address is in the Intranet
or in the Internet respectively.*}

constdefs 
  is_in_intranet :: "IntegerPort  \<Rightarrow> bool"
  "is_in_intranet a \<equiv> (fst a) = 3"

  is_in_internet :: "IntegerPort \<Rightarrow> bool"
  "is_in_internet a \<equiv>  (fst a) >3"


text{*
The next definition is our starting state: an empty trace and the just defined policy.*}

constdefs 
  "\<sigma>_0_ftp" ::  "(IntegerPort, ftp_msg) history \<times>
                 (IntegerPort, ftp_msg) Policy"                                        
  "\<sigma>_0_ftp \<equiv> ([],ftp_policy)"

text{*Next we state the conditions we have on our trace: a normal
behaviour FTP run from the intranet to some server in the internet on
port 21.*}

constdefs "accept_ftp" ::  "(IntegerPort, ftp_msg) history \<Rightarrow> bool"
          "accept_ftp t \<equiv> \<exists> c s i p. t \<in> NB_ftp c s i p \<and> is_in_intranet c \<and> is_in_internet s \<and> (snd s) = 21"



fun packet_with_id where
 "packet_with_id [] i = []"
|"packet_with_id (x#xs) i = (if id x = i then (x#(packet_with_id xs i)) else (packet_with_id xs i))"


text{*The depth of the test case generation corresponds to the maximal
length of generated traces. 4 is the minimum to get a full FTP
protocol run.  *}

testgen_params [depth=5]

profiling_on

text{* The test specification:*}
test_spec "accept_ftp (rev t) \<longrightarrow>
 (\<sigma>_0_ftp \<models> (os  \<leftarrow>  mbind t FTP_ST; (\<lambda> \<sigma>. Some (FUT (rev t) = \<sigma>, \<sigma>))))"
apply(simp add: accept_ftp_def \<sigma>_0_ftp_def)
apply (rule impI)+
apply (unfold NB_ftp_def is_in_internet_def is_in_intranet_def)
apply simp
apply (gen_test_cases "FUT" split: HOL.split_if_asm)
apply (simp_all) 
store_test_thm "ftp_test"


text{* We need to add all required lemmas to the simplifier set, such
that they can be used during test data generation. *}


lemmas ST_simps = Let_def valid_def unit_SE_def bind_SE_def orelse_def
 in_subnet_def src_def dest_def IntegerPort.dest_port_def
 subnet_of_def id_def port_open_def is_init_def is_data_def
 is_port_request_def is_close_def p_accept_def content_def
 PolicyCombinators PortCombinators is_in_intranet_def
 is_in_internet_def intranet_def internet_def exI subnetOf_lemma
 subnetOf_lemma2 subnetOf_lemma3 subnetOf_lemma4 port_open_lemma
 ftp_policy_def

declare ST_simps [simp]

gen_test_data "ftp_test"
declare ST_simps [simp del]


thm ftp_test.test_data

write_clocks "time_stats_FTP.tex"

end
